
def get_all_splex_data(caller, org):
    """
    Get all the data related to Splex for a specific organization.

    Args:
        caller: An object providing the API call functionality.
        org (str): The name of the organization.

    Returns:
        dict: The data related to Splex for the specified organization.
    """
    path = f"plex/org/{org}"
    data, response = caller.r("GET", path)
    return data

def list_all_splex_in_project(caller, project_path):
    '''
    path: org/space/project
    '''
    path = f"asset/list/{project_path}?asset_type=Plex&limit=100&offset=0&sort=c_time%3A-1&search="
    data, response = caller.r("GET", path)
    return data['entries']

def get_splex_binary(caller, sl_path, save_path=None): ## maybe refactor to just get binary data
    """
    Downloads the binary data for a specified snaplex.

    Args:
        caller: The caller object.
        sl_path (str): The path to the snaplex e.g "tidemark-dev/shared/dev1".
        save_path (str, optional): The path to save the downloaded binary data. Default is None.

    Raises:
        AssertionError: If save_path is None or does not end with ".slpropz".

    Returns:
        None
    """
    
    assert save_path != None or save_path.endswith(".slpropz"), "save_path must not be None or end with .slpropz"
    path = f"plex/links/{sl_path}"
    data, response = caller.r("GET", path)
    snap_config_url = data['config']
    caller.download_binary(snap_config_url, save_path)





def get_all_projects(caller, org, project_space):
    """
    Retrieve all projects within a specific organization and project space.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.
        project_space (str): The name of the project space.

    Returns:
        list: A list of all projects in the specified organization and project space.
    """
    path = f"asset/list/{org}/{project_space}?limit=100&offset=0&sort=c_time%3A-1&search="
    data, response = caller.r("GET", path)
    return data['entries']

def get_project(caller, org, project_space, project_name):
    """
    Retrieve the data of a specific project within an organization and project space.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.
        project_space (str): The name of the project space.
        project_name (str): The name of the project.

    Returns:
        dict: The data of the specified project.
    """
    path = f"asset/{org}/{project_space}/{project_name}"
    data, response = caller.r("GET", path)
    return data

def create_project(caller, org, project_space, project_name):
    """
    Create a new project within a specific organization and project space.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.
        project_space (str): The name of the project space.
        project_name (str): The name of the project to be created.

    Returns:
        dict: The data of the newly created project.
    """
    path = f"asset/{org}/{project_space}/{project_name}"
    payload = {
        "asset_type": "Dir",
        "metadata": {
            "pattern": False,
            "validation": True
        },
        "name": project_name
    }
    data, response = caller.r("POST", path, json=payload, headers={'Content-Type': 'application/json'})
    return data

def delete_project(caller, org, project_space, project_name):
    """
    Delete a specific project within an organization and project space.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.
        project_space (str): The name of the project space.
        project_name (str): The name of the project to be deleted.

    Returns:
        dict: The data response after the project is deleted.
    """
    path = f"asset/{org}/{project_space}/{project_name}?soft_delete=True&asset_type=Dir"
    data, response = caller.r("DELETE", path)
    return data


def get_project_space(caller, org, project_space):
    """
    Retrieve the data of a specific project space within an organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.
        project_space (str): The name of the project space.

    Returns:
        dict: The data of the specified project space.
    """

    path = f"asset/{org}/{project_space}"
    data, response = caller.r("GET", path)
    return data

def set_acl(caller, sl_path, subject, role='read', inherit=False, mock=False):
    """
    Set the ACL (Access Control List) for a specified path and subject.

    Parameters:
        caller: The object responsible for making HTTP requests.
        sl_path (str): The path of the asset.
        subject (str): The subject (user or group).
        role (str, optional): The role to be assigned. Defaults to 'read'.
        inherit (bool, optional): Indicates if permissions should be inherited. Defaults to False.
        mock (bool, optional): Indicates if this should be a mock call. Defaults to False.

    Returns:
        None: Sets the ACL for the specified path and subject.
    """

    path = f"asset/acl/{sl_path}"
    
    perms = ["R", "X"]
    if role == 'admin':
        perms = ["R", "W", "X"]
    
    subject_type = "GROUP"
    if "@" in subject:
        subject_type = "USER"
    
    body = {
        "perms": perms,
        "path": f"/{sl_path}",
        "subject": subject,
        "subject_type": subject_type,
        "inherit": inherit
    }
    if mock:
        del body['path']
        return body
    else:
        caller.r("POST", path, json=body, headers={'Content-Type': 'application/json'})


def delete_acl(caller, sl_path, subject):
    """
    Delete the ACL (Access Control List) for a specified path and subject.

    Parameters:
        caller: The object responsible for making HTTP requests.
        sl_path (str): The path of the asset.
        subject (str): The subject (user or group).

    Returns:
        None: Deletes the ACL for the specified path and subject.
    """

    subject_type = "GROUP"
    if "@" in subject:
        subject_type = "USER"
    path = f"asset/acl/{sl_path}"
    query_params = {
        "subject": subject,
        "subject_type": subject_type
    }
    
    caller.r("DELETE", path, query=query_params)

def add_subject_to_project(caller, org, project_space, project_name, subject, ps_role , p_role ):
    """
    Update the ACLs (Access Control Lists) for the project space, project, and shared project to add or update permissions for the specified subject.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.
        project_space (str): The name of the project space.
        project_name (str): The name of the project.
        subject (str): The subject (user or group).
        ps_role (str): The role to be assigned at the project space level.
        p_role (str): The role to be assigned at the project level.

    Returns:
        None: Updates the ACLs for the project space, project, and shared project.
    """
    
    project_space_data = get_project_space(caller, org , project_space)
    project_space_acl = project_space_data['acl']
    project_space_owner = project_space_data['owner']

    project_data = get_project(caller, org, project_space, project_name)
    project_acl = project_data['acl']
    project_owner = project_data['owner']
    
    shared_project_data = get_project(caller, org, project_space, "shared")
    shared_project_acl = shared_project_data['acl']
    shared_project_owner = shared_project_data['owner']

    ## first we do project spaces acl rec
    existing_ps_acl = [acl for acl in project_space_acl if acl['subject'] == subject]
    if len(existing_ps_acl) == 0:
        set_acl(caller, f"{org}/{project_space}", subject, role=ps_role, inherit=False)
    else:
        existing_ps_acl = existing_ps_acl[0]
        existing_ps_acl['perms'].sort()
        generated_acl = set_acl(caller, f"{org}/{project_space}", subject, role=ps_role, inherit=False, mock=True)
        generated_acl['perms'].sort() #type: ignore
        if generated_acl != existing_ps_acl:
            set_acl(caller, f"{org}/{project_space}", subject, role=ps_role, inherit=False)


    ## then we do project acl rec
    existing_p_acl = [acl for acl in project_acl if acl['subject'] == subject]
    if len(existing_p_acl) == 0:
        set_acl(caller, f"{org}/{project_space}/{project_name}", subject, role=p_role, inherit=True)
    else:
        existing_p_acl = existing_p_acl[0]
        existing_p_acl['perms'].sort()
        generated_acl = set_acl(caller, f"{org}/{project_space}/{project_name}", subject, role=p_role, inherit=True, mock=True)
        generated_acl['perms'].sort() #type: ignore
        if generated_acl != existing_p_acl:
            set_acl(caller, f"{org}/{project_space}/{project_name}", subject, role=p_role, inherit=True)

    ## Finally, we check the shared project ACL
    existing_shared_acl = [acl for acl in shared_project_acl if acl['subject'] == subject]
    if len(existing_shared_acl) == 0:
        set_acl(caller, f"{org}/{project_space}/shared", subject, role=ps_role, inherit=True)
    else:
        existing_shared_acl = existing_shared_acl[0]
        existing_shared_acl['perms'].sort()
        shared_generated_acl = set_acl(caller, f"{org}/{project_space}/shared", subject, role=ps_role, inherit=True, mock=True)
        shared_generated_acl['perms'].sort() #type: ignore
        if shared_generated_acl != existing_shared_acl:
            set_acl(caller, f"{org}/{project_space}/shared", subject, role=ps_role, inherit=True)





from rich.table import Table
from tm_sl.com.printing import console, print_json
import json

test_data = [ 
    { "h1":"mydata", "h2":"arg 1" , "h3":"category 4"  } ,
    { "h1":"mydata", "h2":"arg 1" , "h4":"category 4?"  } ,
    { "h1":"mydata", "h2":"arg 1" , "h3": ['l1','l2','l3']  }
    ]

def c_json(data):
    print_json(json.dumps(data))

def json_as_table(data):
    table = Table(show_header=True, header_style="bold magenta")
    
    if isinstance(data, list):
        if len(data) == 0:
            console.print("Empty List!")
            return
        elif all(isinstance(item, dict) for item in data):
            # Get all unique headers across the list of dictionaries
            headers = {key for item in data for key in item.keys()}
            [table.add_column(header) for header in headers]

            for item in data:
                # Get values based on headers, if not exists then use an empty string
                values = [str(item.get(header, '')) for header in headers]
                table.add_row(*values)

            console.print(table)
        else:
            console.print("The list contains non-dict items, can't represent as a table.")
            return
    elif isinstance(data, dict):
        headers = list(data.keys())
        [table.add_column(header) for header in headers]
        table.add_row(*[str(value) for value in data.values()])
        console.print(table)
    else:
        console.print("Unsupported data type, can't represent as a table.")
        return



from tm_sl.com import *

import requests
from requests.auth import HTTPBasicAuth
import os
from dotenv import load_dotenv
from urllib.parse import urlencode


class SnapLogicManager:
    """
    A class representing a SnapLogic Manager.

    Attributes:
        session (requests.Session): The session used for making API requests.
        base (str): The base URL for the API endpoint.
    """
    def __init__(self, user=None, password=None, env=None):
        self.session = self.login(user, password, env)
        self.base = "https://elastic.snaplogic.com/api/1/rest/"

    def __del__(self):
        self.session.close()

    def get_creds(self, user, password, env):
        """
        Get the user credentials.

        Args:
            user (str): The user's username.
            password (str): The user's password.
            env (str): The path to the .env file.

        Returns:
            tuple: The user's username and password.
        """
        if not env and not user and not password:
            # print("Trying to load the default .env file")
            load_dotenv(dotenv_path=".env", override=True)
        elif env:
            # print("Trying to load the .env file from the path provided, ignoring all other parameter inputs")
            load_dotenv(dotenv_path=env, override=True)
        elif not env and (user and password):
            if not user or not password:
                # print("Please provide both username and password")
                exit(1) 
            # print("Got manual username and password")
        # inspect(ctx.obj,all=True)
        ruser = os.getenv('SL_USER') if not user else user
        rpassword = os.getenv('SL_PASSWORD') if not password else password
        return ruser, rpassword

    def login(self, user, password, env):
        """
        Login to the SnapLogic Manager API.

        Args:
            user (str): The user's username.
            password (str): The user's password.
            env (str): The path to the .env file.

        Returns:
            requests.Session: The logged-in session.
        """
        s = requests.Session()
        s.auth = HTTPBasicAuth(*self.get_creds(user, password, env))
         # type: ignore
        return s

    def r(self, method, path, query = None, debug = False,  *args, **kwargs):
        """
        Make a request to the SnapLogic Manager API.

        Args:
            method (str): The HTTP method of the request.
            path (str): The path to the API endpoint.
            query (dict, optional): The query parameters of the request. Defaults to None.
            *args: Additional positional arguments passed to the request method.
            **kwargs: Additional keyword arguments passed to the request method.

        Returns:
            tuple: The response data and the response object.
        """
        response = None
        try:
            url = self.base + path

            if query:
                query_string = urlencode(query)
                url += '?' + query_string

            if method == 'GET': response = self.session.get(url, *args, **kwargs)
            elif method == 'PUT': response = self.session.put(url, *args, **kwargs)
            elif method == 'POST': response = self.session.post(url, *args, **kwargs)
            elif method == 'DELETE': response = self.session.delete(url, *args, **kwargs)
            else: raise Exception(f"Unknown method {method}")
            response.raise_for_status()
            data = response.json()['response_map']
            return data, response
        except Exception as e:
            if debug:
                print(f"[bold red] -- API FAIL {method} {path} --  [/bold red]")
                print(f"[gray]{e}[/gray]")
                if response != None:
                    print(f"Response status code: {response.status_code}")
                    # print(f"Request headers:")
                    # pprint(response.request.headers)
                    if response.request.body: 
                        print(f"Request body:\n{response.request.body}")
                    print(f"[bold red] -- API RESPONSE -- [/bold red]")
                    pprint(response.json())
                print(f"[bold red] -- TRACEBACK -- [/bold red]")
                console.print_exception()
            raise e
        
    def download_binary(self, path, save_path): 
        """
        Download a binary file from the SnapLogic Manager API.

        Args:
            path (str): The path to the binary file.
            save_path (str): The path to save the downloaded file.

        Raises:
            Exception: If the download fails.
        """
        try:
            response = self.session.get( path)
            response.raise_for_status()
            with open(save_path, 'wb') as file:
                file.write(response.content)
        except Exception as e:
            print(f"Failed to download binary file: {e}")



from requests.exceptions import HTTPError

def get_all_users(caller, org):
    """
    Retrieve all users in an org.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name.

    Returns:
        list: A list of all users in the org.
    """
    path = f"asset/user/settings?path=%2F{org}"
    data, response = caller.r("GET", path)
    users = data['users']
    return users

def get_user(caller, email, org=None):
    """
    Retrieve a user by email. If an organization name is provided, it retrieves the user from that specific org.

    Parameters:
        caller: The object responsible for making HTTP requests.
        email (str): The email of the user to retrieve.
        org (str, optional): The organization's name. Defaults to None.

    Returns:
        dict: A dictionary containing the user's details if found.

    Raises:
        Exception: If the user with the given email is not found in the provided org.

    Example:
        >>> user = get_user(caller, "user@example.com", "exampleOrg")
        >>> print(user)
        {'username': 'user@example.com', 'roles': ['admin'], 'status': 'active'}
    """
    if org != None:
        users = get_all_users(caller, org)
        for user in users:
            if user['username'] == email:
                return user
        return None
    path = f"asset/user/{email}"
    data, response = caller.r("GET", path)
    return data

def get_user2(caller, email, org):

    path = f"asset/user/{email}/org/{org}"
    try:
        data, response = caller.r("GET", path)
        return data
    except HTTPError as e:
        if e.response.status_code == 404:
            return None

def create_user(caller, org, email , ip=True):
    """
    Create a new user in a specific organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name where the user will be created.
        email (str): The email of the user to create.
        ip (bool, optional): Intelligent Playform access. Defaults to True.
    Example:
        >>> create_user(caller, "exampleOrg", "newuser@example.com")
        # User 'newuser@example.com' is now created in 'exampleOrg'
    """

    data, response = caller.r(
                            "PUT", 
                            f"asset/user/{email}/org/{org}", 
                            json={"create_home_directory":False}
                            )

 
    payload = {
        "org_path": org,
        "users": [
            {
                "app_access": {
                    "intelligent_integration_platform": ip
                },
                "username": email
            }
        ]
    }
    data, response = caller.r(
                            "POST", 
                            f"asset/user/app_access", 
                            json=payload, 
                            headers={'Content-Type': 'application/json'}
                            )
    if ip:
        data, response = caller.r(
                            "POST", 
                            f"asset/user/app_access", 
                            json=payload, 
                            headers={'Content-Type': 'application/json'}
                            )
        
def delete_user(caller, org, email):
    """
    Delete a user from a specific organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's name from which the user will be deleted.
        email (str): The email of the user to delete.

    Example:
        >>> delete_user(caller, "exampleOrg", "user@example.com")
        # User 'user@example.com' is now deleted from 'exampleOrg'
    """
    path = f"asset/user/{email}/org/{org}"
    data, response = caller.r("DELETE", path)



from tm_sl.com import *

def get_all_tasks(caller, org, project_space, project_name):
    """
    Get all tasks in a project.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        project_space (str): The project space's ID.
        project_name (str): The project's name.

    Returns:
        list: A list of all tasks within the specified project.

    Example:
    ```python
        from tm_sl.sl import tasks as t
        data, response = get_all_tasks(caller, 'myorg', 'myspace', 'myproject')
        print(data)
        # Output: [{'id': '123', 'name': 'task1'}, {'id': '456', 'name': 'task2'}, ...]
    ```
    """
    path = f"asset/list/{org}/{project_space}/{project_name}?asset_type=Job&limit=1000&offset=0&sort=c_time:-1&search="
    data, response = caller.r("GET", path)
    return data['entries']

def get_task_config(caller, snode_id):
    """
    Get the configuration of a task.

    Parameters:
        caller: The object responsible for making HTTP requests.
        snode_id (str): The task's ID.

    Returns:
        dict: The configuration data of the specified task.

    Example:
    ```python
        from tm_sl.sl import tasks as t
        data, response = t.get_task_config(caller, 'ase1234123s123')
        print(data)
        # Output: {'id': '123', 'name': 'task1', 'config': {...}}
    ```
    """
    path = f"slsched/job/{snode_id}?include_rbin=true"
    data, response = caller.r("GET", path)
    return data

def change_snaplex_task(caller, org, snode_id, splex_name):
    """
    Change the snaplex of a task.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        snode_id (str): The task's ID.
        splex_name (str): The name of the snaplex.

    Returns:
        dict: The response data after changing the snaplex of the task.

    Example:
    ```python
        from tm_sl.sl import tasks as t
        data, response = t.change_snaplex_task(caller, 'myorg', '123', 'new_snaplex')
        print(data)
    ```
    """
    task = get_task_config(caller, snode_id)
    try: rtpath = task['parameters']['runtime_path_id']
    except: rtpath = None
    try: tstatus = task['parameters']['enabled']
    except: tstatus = None

    if rtpath == None or rtpath != (org + '/rt/sidekick/'+ splex_name):
        change_one = f"https://elastic.snaplogic.com/api/1/rest/"
        task['parameters']['runtime_path_id'] = org + '/rt/sidekick/'+ splex_name
        if tstatus == None or tstatus != True:
            task['parameters']['enabled'] = True
        # filtered['parameters']['path'] = "/tidemark-us/shared/test"
        data, response = caller.r('PUT', f"slsched/job/{task['snode_id']}?duplicate_check=True" , json=task)
        return data



from tm_sl.sl import groups, projects


def add_tmgroup_to_project(caller, org, project_space, project_name, group ):
    """
    Adds a team group to a project.

    Args:
        caller (str): The caller of the function.
        org (str): The organization name.
        project_space (str): The project space.
        project_name (str): The project name.
        group (str): The team group.

    Raises:
        UnknownError: If an unknown error occurs.

    Returns:
        None
    """
    groups_data = groups.get_all_groups(caller, org)


    subjects = [{'name': f"t_{group}_dev" , 'role': "admin"},{'name': f"t_{group}_run" , 'role': "read"}]
    missing_subjects = []
    for subject in subjects:
        if subject['name'] not in groups_data:
            missing_subjects.append(subject['name'])
    if len(missing_subjects) > 0:
        for s in missing_subjects:
            groups.create_group(caller, org, s)

    for s in subjects:
        projects.add_subject_to_project(caller, org, project_space, project_name, s['name'], 'read', s['role'])



## DOCSTRINGS SHOULD BE GENEREATED WITHOUT PARAMETERS
## DOCSTRINGS ARE NOT REQUIRED FOR GROUP COMMANDS
## AN EXAMPLE USING THE COMMAND NAME IN THIS FILE ALONG WITH THE SUBCOMMAND IS PREFERABLE
## ```shell
##      $ tm-sl --env sl.env group_command subcommand (options)/(args)
##  ```

import click
from tm_sl.com import *

from tm_sl.sl import migration as m




@click.command('migrate', help='Migrate using the v1 API')
@click.option('--src','-s', required=True, help='Source project path in the form of org/project_space/project_name')
@click.option('--dst','-d', required=True, help='Destination project path in the form of org/project_space/project_name')
@click.option('--tasks', '-t', is_flag=True, help='Flag to include tasks in the migration')
@click.option('--pipelines', '-p', is_flag=True, help='Flag to include pipelines in the migration')
@click.option('--files', '-f', is_flag=True, help='Flag to include files in the migration')
@click.option('--accounts', '-a', is_flag=True, help='Flag to include accounts in the migration')
@click.option('--wait', '-w', is_flag=True, help='Follow and wait for the migration to complete')
@click.pass_context
def migrate(ctx, src, dst, tasks, pipelines, files, accounts, wait):
    """
    Migrate assets using the v1 API.

    This command migrates specified types of assets from the source project to the destination project.

    Example:
    ```shell
        $ tm-sl migrate --src myorg/myspace/myproject --dst myorg2/myspace2/myproject2 --tasks --pipelines --wait
        "Migration started. Waiting for completion..."
        "Migration completed successfully."
    ```

    Raises:
        AssertionError: If no asset type is specified for migration.
    """
    assets = []
    if tasks: assets.append("Job")
    if pipelines: assets.append("Pipeline")
    if files: assets.append("File")
    if accounts: assets.append("Account")
    assert len(assets) > 0, "At least one asset type must be specified"

    m.migrate(ctx.obj['manager'] , src, dst, asset_types=assets ,follow=wait)



from . import groups
from . import users
from . import projects
from . import tasks
from . import migration
from . import snaplexes

PACKAGE_NAME = "sl"

__all__ = ['groups', 'users', 'projects', 'tasks', 'migration', 'snaplexes']


import click
import pkg_resources
from tm_sl.com import *
from tm_sl.sl import groups, projects, tasks  # Import your core logic functions
from tm_sl.sl.manager import SnapLogicManager  # Import the SnapLogicManager from where it's defined

from tm_sl.cli.project import project
from tm_sl.cli.group import group
from tm_sl.cli.snaplex import snaplex
from tm_sl.cli.migrate import migrate
from tm_sl.cli.task import task
from tm_sl.cli.tenant import tenant
def _entry_point():
    main(obj={})

@click.group()
@click.pass_context
@click.option('--username', '-u', required=False, default = None, help='SnapLogic username')
@click.option('--password', '-p', required=False, default = None, help='SnapLogic password')
@click.option('--env', '-e', required=False, default = None, help='Path to environment file')
def main(ctx, username, password, env):
    """
    Initialize the CLI application with optional SnapLogic credentials and environment file. 
    It sets up the context for subsequent CLI commands to interact with SnapLogic resources.
    
    If nothing is provided, it will try to load the default .env file in the current directory.
    If a .env file can't be found the env variables SL_USER and SL_PASSWORD will be tried.

    Example:
    ```shell
        $ tm-sl --username myuser --password mypass group list tidemark-dev
        $ tm-sl -e ./slcreds.env group list tidemark-dev
        [
            "admins",
            "members",
            "py_test_dev",
            "py_test_run",
            "python_test_group",
            "t_baseline_devs",
            "t_baseline_run"
        ]
    ```
    """
    ctx.obj['manager'] = SnapLogicManager(username, password, env)  # Create a manager object and store it in the context object

@main.command(name='version')
def version_command():
    """Display the current version of the application."""
    # Replace with your actual version
    version = pkg_resources.get_distribution("tm_sl").version
    print(f"tm-sl version: {version}")

main.add_command(group)
main.add_command(project)
main.add_command(snaplex)
main.add_command(migrate)
main.add_command(task)
main.add_command(tenant)

if __name__ == "__main__":
    _entry_point()




import click
from tm_sl.com import *

from tm_sl.sl import groups, tenants, projects

@click.group('tenant')
@click.pass_context
def tenant(ctx):
    """
    Manage snaplogic functions for the Tidemark application.
    These bindings will include tenant specific combos.
    """
    pass



@tenant.command('prep')
@click.option('--path',"-p", required=True, help="SLPath to the project org/project_space/project")
@click.option('--tenant',"-t", required=True, help="tidemark tenant name e.g. 'teslamotors'")
@click.pass_context
def assert_project_group(ctx, path, tenant):
    """
    Prepares a project, group, and permissions for a tenant.

    1) Create a project if it does not already exist
    2) Create the group if it does not already exist
    3) Add the group to the project if it is not already added and reconciles permissions
    4) Add the group to the project space as well as the shared project in the selected project space

    """
    
    org, project_space, project_name = path.split('/')
    all_projects = projects.get_all_projects(ctx.obj['manager'], org, project_space)
    if project_name not in [project['name'] for project in all_projects]:
        print("[bold blue]Project does not exist, creating...[/bold blue]")
        projects.create_project(ctx.obj['manager'], org, project_space, project_name)
    tenants.add_tmgroup_to_project(ctx.obj['manager'], org, project_space, project_name, tenant)




import click
from tm_sl.com import *

from tm_sl.sl import tasks as t

@click.group('task', help='Manage Tasks.')
@click.pass_context
def task(ctx):
    pass  # Here for the purpose of defining a group, no action needed


@task.command('get_all', help='List all tasks.')
@click.option('--path','-p', required=True, help= "Snaplogic path in the form of org/project_space/project_name")
@click.pass_context
def get_all(ctx, path):
    """
    List all projects.

    This command retrieves and displays all tasks present in the specified Snaplogic path.

    Example:
    ```shell
        $ tm-cli task get_all --path org/project_space/project_name
        [
            "task1",
            "task2",
            "task3"
        ]
    ```
    """
    c_json(t.get_all_tasks(ctx.obj['manager'] , *path.split("/")))

def get_task_config(ctx, path):
    pass

@task.command('fix_all', help='Change the snaplex on all tasks')
@click.option('--path','-p', required=True, help= "Snaplogic path in the form of org/project_space/project_name")
@click.option('--snaplex','-s', required=True, help= "Snaplex name in the 'environment' field of the snaplex config")
@click.pass_context
def fix_all(ctx, path, snaplex):
    """
    Fix all tasks to a new snaplex.

    This command changes the snaplex on all tasks present in the specified Snaplogic path to a new snaplex.

    Example:
    ```shell
        $ tm-sl task fix_all --path org/project_space/project_name --snaplex new_snaplex
        "All tasks have been updated to use the new_snaplex snaplex."
    ```
    """
    org, ps, project = path.split("/")
    data = t.get_all_tasks(ctx.obj['manager'] , *path.split("/") )
    for task in data:
        t.change_snaplex_task(ctx.obj['manager'] , org, task['snode_id'], snaplex)
    


def get_all_groups(caller, org):
    """
    Retrieve all groups within a specified organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.

    Returns:
        list: A list of all groups within the specified organization.
    """
    path = f"asset/group/org/{org}"
    data, response = caller.r("GET", path)
    return data

def get_all_users_in_group(caller, org, group):
    """
    Retrieve all users within a specified group and organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        group (str): The group's ID.

    Returns:
        list: A list of all users within the specified group and organization.
    """
    path = f"asset/group/{group}/org/{org}"
    data, response = caller.r("GET", path)
    return data

def create_group(caller, org, group):
    """
    Create a new group within a specified organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        group (str): The name of the new group to be created.

    Returns:
        dict: The response data after creating the new group.
    """
    path = f"asset/group/org/{org}"
    data = {"groupname": f"{group}", "members": []}
    data, response = caller.r("POST", path, json=data , headers={'Content-Type': 'application/json'})
    return data

def delete_group(caller, org, group):
    """
    Delete a specified group within an organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        group (str): The ID of the group to be deleted.

    Returns:
        dict: The response data after deleting the group.
    """
    path = f"asset/group/{group}/org/{org}"
    data, response = caller.r("DELETE", path)
    return data

def add_users_to_group(caller, org, group, users=[]):
    """
    Add a list of users to a specified group within an organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        group (str): The group's ID.
        users (list, optional): A list of user IDs to be added to the group. Defaults to an empty list.

    Returns:
        dict: The response data after adding users to the group.

    Raises:
        AssertionError: If the users parameter is not a list or is an empty list.
    """
    assert type(users) == list, "Users must be a list"
    assert len(users) > 0, "Users must be a list of length > 0"
    group_users = get_all_users_in_group(caller, org, group)['members']
    for user in users:
        if user not in group_users:
            group_users.append(user)

    path = f"asset/group/{group}/org/{org}"
    data, response = caller.r("PUT", path, json={"members": group_users}, headers={'Content-Type': 'application/json'})
    return data

def overwrite_users_to_group(caller, org, group, users):
    """
    Overwrite the list of users in a specified group within an organization.

    Parameters:
        caller: The object responsible for making HTTP requests.
        org (str): The organization's ID.
        group (str): The group's ID.
        users (list): A new list of user IDs to overwrite the existing users in the group.

    Returns:
        dict: The response data after overwriting users in the group.
    """
    path = f"asset/group/{group}/org/{org}"
    data, response = caller.r("PUT", path, json={"members": users}, headers={'Content-Type': 'application/json'})
    return data



## DOCSTRINGS SHOULD BE GENEREATED WITHOUT PARAMETERS
## DOCSTRINGS ARE NOT REQUIRED FOR GROUP COMMANDS
## AN EXAMPLE USING THE COMMAND NAME IN THIS FILE ALONG WITH THE SUBCOMMAND IS PREFERABLE
## ```shell
##      $ tm-sl --env sl.env group_command subcommand (options)/(args)
##  ```

import click
from tm_sl.com import *

from tm_sl.sl import projects

@click.group('project', help='Manage projects.')
@click.pass_context
def project(ctx):
    pass  # Here for the purpose of defining a group, no action needed


@project.command('list')
@click.argument('org', required=True)
@click.argument('project_space', required=True)
@click.pass_context
def project_list(ctx, org, project_space):
    """
    List all projects within a specified organization and project space.

    This command retrieves and displays all projects present in the specified organization and project space.

    Example:
    ```shell
        $ tm-sl project list myorg myprojectspace
        [
            {
                "name": "Project1",
                "id": "123",
                "status": "active"
            },
            {
                "name": "Project2",
                "id": "456",
                "status": "inactive"
            }
        ]
    ```
    """
    c_json(projects.get_all_projects(ctx.obj['manager'] , org , project_space)) # Call the core logic function




import click
from tm_sl.com import *

from tm_sl.sl import groups, users

@click.group('group')
@click.pass_context
def group(ctx):
    '''
    Manage groups.
    '''
    pass


@group.command('list')
@click.argument('org', required=True)
@click.pass_context
def list_groups(ctx, org):
    """
    List all groups within the specified organization. 
    It retrieves all the groups in an organization and pretty prints them to the console.

    Example:
    ```shell
        $ tm-sl group list tidemark-dev
        [
            "admins",
            "members",
            "py_test_dev",
            "py_test_run",
            "python_test_group",
            "t_baseline_devs",
            "t_baseline_run"
        ]
    ```
    """
    c_json(groups.get_all_groups( ctx.obj['manager'], org))

@group.command('add-user')
@click.argument('org', required=True)
@click.argument('email', required=True)
@click.argument('group_name', required=True)
@click.option('--ip', required=False, is_flag=True, help="Add user to IntelligentPlatform")
@click.pass_context
def add_user(ctx, org, email, group_name, ip):
    """
    Add a user to a group.

    Args:
        ctx (Context): The click Context object.
        org (str): The organization to add the user to.
        email (str): The email address of the user to add.
        group_name (str): The name of the group to add the user to.
        ip (bool): Flag indicating whether to add the user to IntelligentPlatform.

    Returns:
        None

    Raises:
        None
    """
    m = ctx.obj['manager']
    if users.get_user(m, email , org):
        print(f"User {email} already exists")
    else:
        users.create_user(m, org, email)
        print(f"Created user {email}") 

    if email not in groups.get_all_users_in_group(m, org, group_name)['members']:
        groups.add_users_to_group(m, org, group_name, [email])




from tm_sl.com import *
from time import sleep
import re

def validate_project_path(path):
    """
    Validate the format of a project path.

    Args:
        path (str): The project path to be validated.

    Raises:
        ValueError: If the project path has an invalid format.
    """
    pattern = re.compile(r'^[\w-]+/[\w-]+/[\w-]+$')
    if not pattern.match(path):
        raise ValueError(f"Invalid project path format: {path}. Expected format is org/project_space/project_name.")



def migrate(caller, src_path , dst_path, asset_types=["Pipeline","Job"], follow=True):
    """
    Migrate assets from one project to another.

    Args:
        caller: The caller object for making API requests.
        src_path (str): The source path of the project to migrate from.
        dst_path (str): The destination path of the project to migrate to.
        asset_types (list, optional): A list of asset types to migrate. Defaults to ["Pipeline","Job"].
        follow (bool, optional): Whether to wait for the migration to complete. Defaults to True.

    Returns:
        dict: The response data from the migration request.
    """


    validate_project_path(src_path)
    validate_project_path(dst_path)

    all_assets = ["Account","Pipeline","Job","File"]
    assert all([x in all_assets for x in asset_types]), f"asset_types must be a subset of {all_assets}"


    path = "public/project/migrate/"+src_path
    payload={
        "dest_path":"/"+dst_path,
        "asset_types":asset_types,
        "async_mode": True,
        "duplicate_check":"false"}
    print(f"[bold blue]Migrating [/bold blue]")
    c_json(asset_types)
    print(f"[blue]from: [/blue]{src_path}")
    print(f"[blue]to:   [/blue]{dst_path}")

    data, response = caller.r("POST", path, json=payload, headers={'Content-Type': 'application/json'})
    c_json(data)

    if follow:
        print(f"[dark_orange]Follow was selected, waiting for migration to complete[/dark_orange]")
        fpath = data['status_url'].replace('https://elastic.snaplogic.com/api/1/rest/', '')
        counter = 0
        # print(caller.r("GET", fpath))
        with console.status("[bold green]Migrating...[/bold green]", spinner="dots"):
            while caller.r("GET", fpath)[0]['status'] == 'Started':
                sleep(10)
                counter += 1
                if counter > 220: break
        print(f"[bold green]Migration reported as completed[/bold green]")
    else:
        print(f"[bold blue]Follow was not selected, migration will complete asyncronously[/bold blue]")
    return data



import click
from tm_sl.com import *

from tm_sl.sl import snaplexes as s

@click.group('snaplex')
@click.pass_context
def snaplex(ctx):
    '''
    Manage Snaplexes.
    '''
    pass  # Here for the purpose of defining a group, no action needed


@snaplex.command('list')
@click.argument('org', required=True)
@click.pass_context
def snaplex_list(ctx, org):
    """
    List all Snaplexes in a specified organization.

    This command retrieves and displays all Snaplexes present in the specified organization.

    Example:
    ```shell
        $ tm-sl snaplex get_all myorg
        [
            {
                "name": "Snaplex1",
                "id": "123",
                "status": "active"
            },
            {
                "name": "Snaplex2",
                "id": "456",
                "status": "inactive"
            }
        ]
    ```
    """
    c_json(s.get_all_splex_data(ctx.obj['manager'] , org))

@snaplex.command('binary')
@click.option('--path','-p', required=True, help="Snaplex path in the form of org/project_space/project_name/snaplex_name")
@click.option('--savepath','-s', required=True, help="Filepath to save the snaplex binary to (must end with .slpropz)")
@click.pass_context
def snaplex_binary(ctx, path, savepath):
    """
    Downloads a binary config of the specified snaplex to the specified filepath.
    """
    s.get_splex_binary(ctx.obj['manager'] , path, savepath)